!------------------------------------------------------------------------------ 
! Copyright (c) 2008 Pascal Gentil
!
! Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
! les termes de la Licence Publique Générale GNU telle que publiée par la Free
! Software Foundation ; soit la version 2 de cette licence, soit (à vous de
! voir...) toute autre version ultérieure.
!
! Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE
! GARANTIE, ni explicite ni implicite, y compris les garanties de
! commercialisation ou d'adaptation dans un but spécifique. Prenez
! connaissance de la Licence Publique Générale GNU pour plus de détails.
! 
! Vous devez avoir reçu une copie de la Licence Publique Générale GNU avec ce
! programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation,
! Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!------------------------------------------------------------------------------ 
program f90tolatex
!------------------------------------------------------------------------------ 
! On réécrit un fichier source Fortran 90/95 en LaTeX avec les mots clés 
! en gras, les commentaires en italiques et les chaînes de caractères en 
! fonte sans empattement
!
! Date de création : 10 juillet 2007	
! Dernière modification : 13 août 2008
!------------------------------------------------------------------------------
! Chargement des modules
!------------------------------------------------------------------------------
  use mod_definitions
  use mod_init_liste_mots_cles
  use mod_nomtolatex
  use mod_entete
  use mod_traduction
  implicit none
!------------------------------------------------------------------------------
! Variables
!------------------------------------------------------------------------------
! Statut du fichier (présence, fin, allocation, ...)
  integer :: etat

! Nom du fichier d'entrée
  character (len = 80) :: nom_entree

! Nom du fichier d'entrée sans son extension (i.e basename)
  character (len = 70) :: nom_court

! Extension du nom du fichier d'entrée (i.e nom_entree - nom_court)
  character (len = 10) :: extension

! Nom du fichier d'entrée traduit en LaTeX
  character (len = 80) :: nom_latex

! Nom du fichier de sortie
  character (len = 80) :: nom_sortie

! Ligne lue en entrée. 132 caractères max
  character (len = 132) :: ligne

! Fonction qui  calcule le nombre de mots-clés 
  integer calcul_mots_cles

! Nombre de mots-clés définis
  integer :: nb_mots_cles

! Tableau dynamique contenant les mots clés
  character (len = MOT_MAX), dimension (:), allocatable :: liste_mots_cles

!------------------------------------------------------------------------------
! Présentation (affichage du logo)
  call presentation ()

do

  write (*, 11, advance = "no") &
  "Nom du fichier source Fortran 90/95 [<CR> pour quitter] : "
  read (*, 11) nom_entree

! On sort en cas d'entrée vide
  if (len_trim(nom_entree) == 0) stop

! Ouverture du fichier d'entrée
  open (UN_ENTREE, file = nom_entree, form = "formatted", status = "old", &
        action = "read", iostat = etat)

  if (etat /= 0) then
    write (*,12) "===> Erreur : le fichier ",  &
    nom_entree(1:len_trim(nom_entree)), " est introuvable."
    print *
    cycle
  end if

! On récupère nom_court et extension pour vérfier le type et créer le nom_latex
  nom_court = nom_entree (1:index(nom_entree,'.') - 1)
  extension = nom_entree (index(nom_entree, '.') + 1:len_trim(nom_entree))

  if (extension /= 'f' .AND. extension /= "f90" .AND. extension /= 'f95') then
    write (*,11) "===> Erreur : ce fichier ne semble pas être du Fortran 90/95"
    print *
    cycle
  else
    exit 
  end if
 
end do

! Ouverture du fichier de sortie 
  nom_sortie = nom_court(1:len_trim(nom_court)) // ".tex"

  write (*,13) "===> Traduction de ", nom_entree(1:len_trim(nom_entree)), &
               " en ", nom_sortie(1:len_trim(nom_sortie))

  open (UN_SORTIE, file = nom_sortie, form = "formatted", status = "unknown", &
        action = "write")

! Calcul du nombre de mots clés définis
  nb_mots_cles = calcul_mots_cles ()

! Initialisation du tableau contenant les mots clés
  allocate (liste_mots_cles(nb_mots_cles), stat=etat)
  if (etat /= 0) then
    write (*,11) "===> Erreur d'allocation du tableau de mots-clés"
    stop
  end if
  call init_liste_mots_cles (nb_mots_cles, liste_mots_cles)

! Définition du nom du fichier Fortran dans le fichier TeX 
  call nomtolatex (nom_entree, nom_latex)

! En tête du fichier
  call entete (nom_latex)

! Boucle de lecture et de traduction
  do 
    read (UN_ENTREE, 11, iostat = etat) ligne 
    if (etat /= 0) exit
    call traduction (ligne, liste_mots_cles, nb_mots_cles)
  end do

! Désallocation du tableau contenant les mots clés
  deallocate (liste_mots_cles)

! Fermeture du fichier d'entrée
  close (UN_ENTREE)

! Fin du fichier de sortie
  write (UN_SORTIE, 11) BACKSL // "end{document}"

! Fermeture du fichier de sortie
  close (UN_SORTIE)

11 format (A)
12 format (A,A,A)
13 format (A,A,A,A)

end program f90tolatex
