!------------------------------------------------------------------------------ 
! Copyright (c) 2008 Pascal Gentil
!
! Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
! les termes de la Licence Publique Générale GNU telle que publiée par la Free
! Software Foundation ; soit la version 2 de cette licence, soit (à vous de
! voir...) toute autre version ultérieure.
!
! Ce programme est distribué dans l'espoir qu'il sera utile, mais SANS AUCUNE
! GARANTIE, ni explicite ni implicite, y compris les garanties de
! commercialisation ou d'adaptation dans un but spécifique. Prenez
! connaissance de la Licence Publique Générale GNU pour plus de détails.
! 
! Vous devez avoir reçu une copie de la Licence Publique Générale GNU avec ce
! programme ; si ce n'est pas le cas, écrivez à la Free Software Foundation,
! Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!------------------------------------------------------------------------------ 
! Module : mod_traduction
! Contenu : routine traduction 
! Date de création : 15 juillet 2008
! Dernière modification : 13 août 2008 
!------------------------------------------------------------------------------ 
module mod_traduction
contains
!------------------------------------------------------------------------------ 
! Fonction : traduction d'une ligne
!
! Entrée :
!  entree : ligne lue dans le fichier source 
!  liste_mots_cles : tableau dynamique contenant les mots clés
!  nb_mots_cles : nombre d'éléments du tableau liste_motcle
!
! Sortie : 
!
! Local : 
!  c : un caractère de la chaine d'entrée
!  chaine : est-on ou non déjà dans une chaine ?
!  ind_entree : index de la chaine d'entrée
!  ind_sortie : index de la chaine de sortie
!  long_entree : longueur réelle de la chaine d'entrée
!  mot : mot lu caractère par caractère
!  sortie : chaine de sortie (i.e entree traduite)
!
! Note : 
! On traduit ligne par ligne, caractère par caractère
!------------------------------------------------------------------------------
subroutine traduction (entree, liste_mots_cles, nb_mots_cles)

  use mod_definitions
  use mod_caractere
  use mod_recherche_mot_cle
  use mod_ecrire_indentation
  use mod_ecrire_commentaire
  use mod_ecrire_chaine
  use mod_ecrire_mot_cle
  use mod_ecrire_mot
  use mod_special  

  implicit none

  character (len = *), intent(in) :: entree
  integer, intent(in) :: nb_mots_cles
  character (len = MOT_MAX), dimension (:), intent(in) :: liste_mots_cles
   
  character c
  logical chaine
  integer ind_entree, ind_sortie, long_entree, fin
  character (len = MOT_MAX + 1) mot
  character (len =  500) sortie

  long_entree = len_trim (entree)
  ind_entree = 0
  ind_sortie = 1
  chaine = .FALSE.
  sortie =  ' '

! Est-ce une ligne blanche ? 
  if (long_entree == 0) then
    write (UN_SORTIE, 11) BACKSL // "medskip"
    return
  end if  

! Boucle générale
  do while (ind_entree <= long_entree)
    ind_entree = ind_entree + 1

! On lit la chaine d'entrée caractère par caractère
    c = entree(ind_entree:ind_entree)       

! Si la ligne commence par un blanc (indentation) on écrit l'indentation
    if (ind_entree == 1 .and. c == ' ') then
      do while (entree(ind_entree:ind_entree) == ' ') 
        ind_entree = ind_entree + 1
      end do
      ind_entree = ind_entree - 1
      call ecrire_indentation (ind_entree, sortie, ind_sortie)
    end if

! Est-ce un commentaire
    if (c == '!') then
      call ecrire_commentaire (entree, ind_entree, long_entree, sortie, &
                               ind_sortie)
      write (UN_SORTIE,11) sortie(1:ind_sortie)
      write (UN_SORTIE,11)
      return
    end if  

! Est-ce un délimiteur de chaine de caractères ?
    if (c == '''' .or. c == '"') then
      call ecrire_chaine (c, entree, ind_entree, long_entree, sortie, &
                          ind_sortie) 
      cycle
    end if

! Cas general 
    if (caractere(c)) then      
      fin = ind_entree + 1
      do
        c = entree(fin:fin)
        if (.NOT. caractere(c)) exit
        fin = fin + 1
      end do
      mot = entree(ind_entree : fin - 1)
      ind_entree = fin - 1

      if (recherche_mot_cle (mot, liste_mots_cles, nb_mots_cles)) then   
        call ecrire_mot_cle (mot, sortie, ind_sortie)
      else
         call ecrire_mot (mot, sortie, ind_sortie) 
      end if 
      cycle
    else
      call special (c, sortie, ind_sortie)
    end if
  end do

! Enfin on écrit le résultat et on passe une ligne blanche pour forcer le
! changement de ligne en TeX
   write (UN_SORTIE,11) sortie(1:ind_sortie)
   write (UN_SORTIE,11)

! Format
11 format (A)

end subroutine traduction
!------------------------------------------------------------------------------
end module mod_traduction
